local VERSION = tonumber(("$Revision: 31328 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Aura = PitBull:NewModule("Aura", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Aura
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-03-30 00:04:47 -0400 (Fri, 30 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

local GameTooltip = GameTooltip
local UnitBuff = UnitBuff
local UnitDebuff = UnitDebuff
local math_floor = math.floor
local math_min = math.min

PitBull_Aura:RegisterPitBullChildFrames('auraFrame')

local unitsShown = {}

local raidfilter = 0

function PitBull_Aura:OnEnable(first)
	self:RegisterBucketEvent("UNIT_AURA", 0.05)
end

function PitBull_Aura:UNIT_AURA(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateAuras(unit, frame)
		end
	end
end

local function buffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:SetUnitBuff(this.unit, this.id, raidfilter)
end
buffFrame_OnEnter = wrap(buffFrame_OnEnter, 'Aura/buffFrame_OnEnter')

local function debuffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:SetUnitDebuff(this.unit, this.id, raidfilter)
end
debuffFrame_OnEnter = wrap(debuffFrame_OnEnter, 'Aura/debuffFrame_OnEnter')

local function buffFrame_OnLeave(this)
	GameTooltip:Hide()
end
buffFrame_OnLeave = wrap(buffFrame_OnLeave, 'Aura/buffFrame_OnLeave')
local debuffFrame_OnLeave = buffFrame_OnLeave

local maxbuffs, maxdebuffs = 32, 40
function PitBull_Aura:UpdateAuras(unit, frame)
	if not unitsShown[unit] then
		return
	end
	local buffs = frame.buffs or new()
	local debuffs = frame.debuffs or new()
	
	for i = 1, maxbuffs do
		local texture, applications
		
		_, _, texture, applications = UnitBuff(unit, i, raidfilter)
		if not texture then
			for j = #buffs, i, -1 do
				local v = buffs[j]
				buffs[j] = nil
				v.text = delFrame(v.text)
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v:SetScript("OnEnter", nil)
				v:SetScript("OnLeave", nil)
				delFrame(v)
			end
			break
		end
		
		local buffFrame = buffs[i] or newFrame("Button", frame)
		buffFrame:ClearAllPoints()
		buffFrame:SetWidth(16)
		buffFrame:SetHeight(16)
		local text = buffFrame.text or newFrame("FontString", buffFrame, "OVERLAY")
		buffFrame.text = text
		text:SetFontObject(GameFontHighlightSmall)
		if applications <= 1 then
			text:SetText("")
		else
			text:SetText(applications)
		end
		text:SetPoint("BOTTOMRIGHT", 1, 0)
		local icon = buffFrame.icon or newFrame("Texture", buffFrame, "BACKGROUND")
		buffFrame.icon = icon
		icon:SetTexture(texture)
		icon:SetAllPoints(buffFrame)
		buffFrame:SetScript("OnEnter", buffFrame_OnEnter)
		buffFrame:SetScript("OnLeave", buffFrame_OnLeave)
		buffFrame.id = i
		buffFrame.unit = unit
		buffs[i] = buffFrame
	end
	
	for i = 1, maxdebuffs do
		local texture, applications, dispelType
		
		_, _, texture, applications, dispelType = UnitDebuff(unit, i, raidfilter)
		if not texture then
			for j = #debuffs, i, -1 do
				local v = debuffs[j]
				debuffs[j] = nil
				v.text = delFrame(v.text)
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v:SetScript("OnEnter", nil)
				v:SetScript("OnLeave", nil)
				delFrame(v)
			end
			break
		end
		
		local debuffFrame = debuffs[i] or newFrame("Button", frame)
		debuffFrame:ClearAllPoints()
		debuffFrame:SetWidth(16)
		debuffFrame:SetHeight(16)
		local text = debuffFrame.text or newFrame("FontString", debuffFrame, "OVERLAY")
		debuffFrame.text = text
		text:SetFontObject(GameFontHighlightSmall)
		if applications <= 1 then
			text:SetText("")
		else
			text:SetText(applications)
		end
		text:SetPoint("BOTTOMRIGHT", 1, 0)
		local icon = debuffFrame.icon or newFrame("Texture", debuffFrame, "BACKGROUND")
		debuffFrame.icon = icon
		icon:SetTexture(texture)
		icon:SetAllPoints(debuffFrame)
		debuffFrame:SetScript("OnEnter", debuffFrame_OnEnter)
		debuffFrame:SetScript("OnLeave", debuffFrame_OnLeave)
		debuffFrame.id = i
		debuffFrame.unit = unit
		debuffs[i] = debuffFrame
	end
	
	local num_buffs = #buffs
	local num_debuffs = #debuffs
	
	local max_auras = math_floor(frame.auraFrame:GetWidth() / 16)
	
	local num_shown_buffs = math_min(num_buffs, max_auras)
	local num_shown_debuffs = math_min(num_debuffs, max_auras - num_shown_buffs)
	
	for i = 1, num_shown_buffs do
		local buff = buffs[i]
		buff:Show()
		buff:ClearAllPoints()
		buff:SetPoint("TOPLEFT", frame.auraFrame, "TOPLEFT", (i-1) * 16, 0)
	end
	for i = num_shown_buffs + 1, num_buffs do
		local buff = buffs[i]
		buff:Hide()
		buff:ClearAllPoints()
	end
	for i = 1, num_shown_debuffs do
		local debuff = debuffs[i]
		debuff:Show()
		debuff:ClearAllPoints()
		debuff:SetPoint("TOPRIGHT", frame.auraFrame, "TOPRIGHT", (i-1) * -16, 0)
	end
	for i = num_shown_debuffs + 1, num_debuffs do
		local debuff = debuffs[i]
		debuff:Hide()
		debuff:ClearAllPoints()
	end
	
	if num_buffs == 0 then
		buffs = del(buffs)
	end
	frame.buffs = buffs
	if num_debuffs == 0 then
		debuffs = del(debuffs)
	end
	frame.debuffs = debuffs
end
PitBull_Aura.OnUpdateFrame = PitBull_Aura.UpdateAuras

function PitBull_Aura:OnPopulateUnitFrame(unit, frame)
	frame.auraFrame = newFrame("Frame")
end

function PitBull_Aura:ClearAuras(unit, frame)
	local buffs = frame.buffs
	if buffs then
		for i = #buffs, 1, -1 do
			local v = buffs[i]
			buffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			v.unit = nil
			v.id = nil
			v:SetScript("OnEnter", nil)
			v:SetScript("OnLeave", nil)
			delFrame(v)
		end
		frame.buffs = del(buffs)
	end

	local debuffs = frame.debuffs
	if debuffs then
		for i = #debuffs, 1, -1 do
			local v = debuffs[i]
			debuffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			v.unit = nil
			v.id = nil
			v:SetScript("OnEnter", nil)
			v:SetScript("OnLeave", nil)
			delFrame(v)
		end
		frame.debuffs = del(debuffs)
	end
end

function PitBull_Aura:OnClearUnitFrame(unit, frame)
	self:ClearAuras(unit, frame)
	
	frame.auraFrame = delFrame(frame.auraFrame)
end

function PitBull_Aura:OnUnknownLayout(unit, frame, name)
	frame.auraFrame:SetHeight(16)
	frame.auraFrame:SetPoint("TOPLEFT", frame, "BOTTOMLEFT")
	frame.auraFrame:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT")
end

function PitBull_Aura:OnUpdateLayout(unit, frame)
	local old = not unitsShown[unit]
	local now = not frame.auraFrame:IsShown()
	if old == now then
		return
	end
	unitsShown[unit] = not now
	if now then
		self:ClearAuras(unit, frame)
	else
		self:UpdateAuras(unit, frame)
	end
end

wrap(PitBull_Aura, 'Aura')
